import { paramsToStringArray, throwError, } from './sqlite.common';
const iosProperty = (_self, property) => {
    if (typeof property === 'function') {
        return property.call(_self);
    }
    else {
        return property;
    }
};
const toCharPtr = (str) => {
    const objcStr = NSString.stringWithString(str);
    const size = strlen(objcStr.UTF8String) + 1;
    const buffer = interop.alloc(size);
    objcStr.getCStringMaxLengthEncoding(buffer, size, NSUTF8StringEncoding);
    return buffer;
};
const getNewCursorStatement = (statement) => ({
    statement,
    built: false,
    columns: [],
    count: undefined,
});
function getResultsAsObject(cursorSt, transformBlobs) {
    const data = {};
    const dict = cursorSt.resultDictionary;
    dict.enumerateKeysAndObjectsUsingBlock((key, value) => {
        if (transformBlobs && value instanceof NSData) {
        }
        else {
            data[key] = value;
        }
    });
    return data;
}
;
function getResultsAsArray(cursorSt, transformBlobs) {
    const data = [];
    const dict = cursorSt.resultDictionary;
    dict.enumerateKeysAndObjectsUsingBlock((key, value) => {
        if (transformBlobs && value instanceof NSData) {
        }
        else {
            data.push(value);
        }
    });
    return data;
}
;
function getRealPath(dbname, create = false) {
    if (dbname === ':memory:') {
        return null;
    }
    return dbname;
}
function getRaw(db, query, params, asObject, transformBlobs) {
    const s = db.executeQueryWithArgumentsInArray(query, paramsToStringArray(params));
    if (s) {
        return asObject ? getResultsAsObject(s, transformBlobs) : getResultsAsArray(s, transformBlobs);
    }
    else {
        throw db.lastError();
    }
}
function eachRaw(db, query, params, asObject, callback, complete, transformBlobs) {
    const getResults = asObject ? getResultsAsObject : getResultsAsArray;
    return Promise.resolve()
        .then(() => {
        let count = 0;
        const s = db.executeQueryWithArgumentsInArray(query, paramsToStringArray(params));
        if (s) {
            while (s.next()) {
                const row = getResults(s, transformBlobs);
                if (row) {
                    count++;
                    callback(null, row);
                }
            }
        }
        else {
            throw db.lastError();
        }
        complete && complete(null, count);
        return count;
    })
        .catch(err => {
        const errorCB = complete || callback;
        if (errorCB) {
            errorCB(err, null);
        }
        return Promise.reject(err);
    });
}
function selectRaw(db, query, params, asObject, transformBlobs) {
    let rows = [];
    const getResults = asObject ? getResultsAsObject : getResultsAsArray;
    const s = db.executeQueryWithArgumentsInArray(query, paramsToStringArray(params));
    if (s) {
        while (s.next()) {
            const row = getResults(s, transformBlobs);
            if (row) {
                rows = [...rows, row];
            }
        }
    }
    else {
        throw db.lastError();
    }
    return rows;
}
function execRaw(db, query, params) {
    const s = db.executeUpdateWithArgumentsInArray(query, paramsToStringArray(params));
    if (!s) {
        throw db.lastError();
    }
}
async function transactionRaw(db, action, isFirstTransaction) {
    try {
        if (isFirstTransaction) {
            execRaw(db, 'BEGIN EXCLUSIVE TRANSACTION');
        }
        const cancelled = { value: false };
        const cancel = () => {
            cancelled.value = true;
        };
        const result = await action(cancel);
        if (!cancelled.value && isFirstTransaction) {
            execRaw(db, 'COMMIT TRANSACTION');
        }
        else if (cancelled.value && isFirstTransaction) {
            execRaw(db, 'ROLLBACK TRANSACTION');
        }
        return result;
    }
    catch (e) {
        if (isFirstTransaction) {
            execRaw(db, 'ROLLBACK TRANSACTION');
        }
        throwError(`transaction: ${e}`);
        return null;
    }
}
export class SQLiteDatabase {
    constructor(filePath, options) {
        this.filePath = filePath;
        this.isOpen = false;
        this._isInTransaction = false;
        this.transformBlobs = !options || options.transformBlobs !== false;
    }
    async open() {
        if (!this.db) {
            this.db = FMDatabase.databaseWithPath(getRealPath(this.filePath));
        }
        if (!this.isOpen) {
            this.isOpen = this.db.open();
        }
        return this.isOpen;
    }
    async close() {
        if (!this.isOpen)
            return;
        this.db.close();
        this.db = null;
        this.isOpen = false;
    }
    async setVersion(version) {
        this.db.userVersion = version + 0;
    }
    async getVersion() {
        return this.db.userVersion;
    }
    async execute(query, params) {
        return execRaw(this.db, query, params);
    }
    async get(query, params, transformBlobs) {
        return (getRaw(this.db, query, params, true, transformBlobs !== null && transformBlobs !== void 0 ? transformBlobs : this.transformBlobs) || null);
    }
    async getArray(query, params, transformBlobs) {
        return (getRaw(this.db, query, params, false, transformBlobs !== null && transformBlobs !== void 0 ? transformBlobs : this.transformBlobs) || null);
    }
    async select(query, params, transformBlobs) {
        return selectRaw(this.db, query, params, true, transformBlobs !== null && transformBlobs !== void 0 ? transformBlobs : this.transformBlobs);
    }
    async each(query, params, callback, complete, transformBlobs) {
        return eachRaw(this.db, query, params, true, callback, complete, transformBlobs !== null && transformBlobs !== void 0 ? transformBlobs : this.transformBlobs);
    }
    async selectArray(query, params, transformBlobs) {
        return selectRaw(this.db, query, params, false, transformBlobs !== null && transformBlobs !== void 0 ? transformBlobs : this.transformBlobs);
    }
    async transaction(action) {
        let res;
        if (!this._isInTransaction) {
            this._isInTransaction = true;
            res = transactionRaw(this.db, action, true);
            this._isInTransaction = false;
        }
        else {
            res = transactionRaw(this.db, action, false);
        }
        return res;
    }
}
export function openOrCreate(filePath, flags, options) {
    const obj = new SQLiteDatabase(getRealPath(filePath), options);
    obj.open();
    return obj;
}
export const deleteDatabase = (filePath) => {
    filePath = getRealPath(filePath);
    const fileManager = iosProperty(NSFileManager, NSFileManager.defaultManager);
    if (fileManager.fileExistsAtPath(filePath)) {
        return fileManager.removeItemAtPathError(filePath);
    }
    return false;
};
//# sourceMappingURL=sqlite.ios.js.map