import { paramsToStringArray, throwError } from './sqlite.common';
export function byteArrayToBuffer(value) {
    if (!value) {
        return null;
    }
    const length = value.length;
    const ret = new Uint8Array(length);
    const isString = typeof value === 'string';
    for (let i = 0; i < length; i++) {
        ret[i] = isString ? value.charCodeAt(i) : value[i];
    }
    return ret;
}
const dataFromCursor = (cursor, transformBlobs) => {
    const colCount = cursor.getColumnCount();
    const data = {};
    for (let i = 0; i < colCount; i++) {
        const type = cursor.getType(i);
        const name = cursor.getColumnName(i);
        switch (type) {
            case android.database.Cursor.FIELD_TYPE_INTEGER:
                data[name] = cursor.getDouble(i);
                break;
            case android.database.Cursor.FIELD_TYPE_FLOAT:
                data[name] = cursor.getDouble(i);
                break;
            case android.database.Cursor.FIELD_TYPE_STRING:
                data[name] = cursor.getString(i);
                break;
            case android.database.Cursor.FIELD_TYPE_BLOB:
                data[name] = transformBlobs ? byteArrayToBuffer(cursor.getBlob(i)) : cursor.getBlob(i);
                break;
            case android.database.Cursor.FIELD_TYPE_NULL:
                data[name] = null;
                break;
            default:
                throwError(`unknown.type: ${type}`);
        }
    }
    return data;
};
const arrayFromCursor = (cursor, transformBlobs) => {
    const colCount = cursor.getColumnCount();
    const data = [];
    for (let i = 0; i < colCount; i++) {
        const type = cursor.getType(i);
        switch (type) {
            case android.database.Cursor.FIELD_TYPE_NULL:
                data.push(null);
                break;
            case android.database.Cursor.FIELD_TYPE_STRING:
                data.push(cursor.getString(i));
                break;
            case android.database.Cursor.FIELD_TYPE_FLOAT:
                data.push(cursor.getDouble(i));
                break;
            case android.database.Cursor.FIELD_TYPE_INTEGER:
                data.push(cursor.getDouble(i));
                break;
            case android.database.Cursor.FIELD_TYPE_BLOB:
                data.push(transformBlobs ? byteArrayToBuffer(cursor.getBlob(i)) : cursor.getBlob(i));
                break;
            case android.database.Cursor.FIELD_TYPE_NULL:
                data.push(null);
                break;
            default:
                throwError(`unknown.type: ${type}`);
        }
    }
    return data;
};
const rawSql = (onCursor) => (db, transformBlobs) => (sql, params) => {
    const parameters = paramsToStringArray(params);
    const cursor = db.rawQuery(sql, parameters);
    try {
        const result = [];
        while (cursor.moveToNext()) {
            result.push(onCursor(cursor, transformBlobs));
        }
        return result;
    }
    finally {
        cursor.close();
    }
};
const eachRaw = (onCursor) => (db, transformBlobs) => (sql, params, callback, complete) => {
    const parameters = paramsToStringArray(params);
    const cursor = db.rawQuery(sql, parameters);
    return Promise.resolve()
        .then(() => {
        const count = 0;
        while (cursor.moveToNext()) {
            const result = onCursor(cursor, transformBlobs);
            callback(null, result);
        }
        cursor.close();
        complete && complete(null, count);
        return count;
    })
        .catch((err) => {
        cursor.close();
        const errorCB = complete || callback;
        if (errorCB) {
            errorCB(err, null);
        }
        return Promise.reject(err);
    });
};
const transactionRaw = async (db, action) => {
    db.beginTransaction();
    try {
        const cancelled = { value: false };
        const cancel = () => {
            cancelled.value = true;
        };
        const result = await action(cancel);
        if (!cancelled.value) {
            db.setTransactionSuccessful();
        }
        return result;
    }
    finally {
        db.endTransaction();
    }
};
const messagePromises = {};
export class SQLiteDatabaseBase {
    constructor(filePath, options) {
        this.filePath = filePath;
        this._isInTransaction = false;
        this.threading = false;
        this.threading = options && options.threading === true;
        this.flags = options === null || options === void 0 ? void 0 : options.flags;
        this.transformBlobs = !options || options.transformBlobs !== false;
    }
    onWorkerMessage(event) {
        const data = event.data;
        const id = data.id;
        if (id && messagePromises.hasOwnProperty(id)) {
            messagePromises[id].forEach(function (executor) {
                executor.timeoutTimer && clearTimeout(executor.timeoutTimer);
                if (data.error) {
                    executor.reject(data.error);
                }
                else {
                    executor.resolve(data.result);
                }
            });
            delete messagePromises[id];
        }
    }
    sendMessageToWorker(nativeData, messageData, timeout = 0) {
        return new Promise((resolve, reject) => {
            let id = Date.now().valueOf();
            if (id <= this.lastId) {
                id = this.lastId + 1;
            }
            this.lastId = id;
            messagePromises[id] = messagePromises[id] || [];
            let timeoutTimer;
            if (timeout > 0) {
                timeoutTimer = setTimeout(() => {
                    try {
                        reject(new Error('timeout'));
                    }
                    catch (_a) { }
                    delete messagePromises[id];
                }, timeout);
            }
            messagePromises[id].push({ resolve, reject, timeoutTimer });
            const keys = Object.keys(nativeData);
            keys.forEach((k) => {
                com.akylas.sqlite.WorkersContext.setValue(`${id}_${k}`, nativeData[k]._native || nativeData[k]);
            });
            const mData = Object.assign({
                type: 'call',
                id,
                dbOptions: {
                    transformBlobs: this.transformBlobs
                },
                nativeDataKeys: keys,
            }, messageData);
            this.worker.postMessage(mData);
        });
    }
    get isOpen() {
        return this.db && this.db.isOpen();
    }
    async close() {
        if (!this.isOpen)
            return;
        if (this.worker) {
            this.worker.postMessage({
                type: 'terminate',
            });
            this.worker = null;
        }
        this.db.close();
        this.db = null;
    }
    async setVersion(version) {
        this.db.setVersion(version);
    }
    async getVersion() {
        return this.db.getVersion();
    }
    async execute(query, params) {
        if (this.threading) {
            return this.sendMessageToWorker({
                db: this.db,
            }, {
                callName: 'execute',
                args: [query, params],
            });
        }
        return this.db.execSQL(query, paramsToStringArray(params));
    }
    async get(query, params, transformBlobs) {
        if (this.threading) {
            return this.sendMessageToWorker({
                db: this.db,
            }, {
                callName: 'get',
                args: [query, params],
            });
        }
        return rawSql(dataFromCursor)(this.db, transformBlobs !== null && transformBlobs !== void 0 ? transformBlobs : this.transformBlobs)(query, params)[0] || null;
    }
    async getArray(query, params, transformBlobs) {
        if (this.threading) {
            return this.sendMessageToWorker({
                db: this.db,
            }, {
                callName: 'getArray',
                args: [query, params],
            });
        }
        return rawSql(arrayFromCursor)(this.db, transformBlobs !== null && transformBlobs !== void 0 ? transformBlobs : this.transformBlobs)(query, params)[0] || null;
    }
    async select(query, params, transformBlobs) {
        if (this.threading) {
            return this.sendMessageToWorker({
                db: this.db,
            }, {
                callName: 'select',
                args: [query, params],
            });
        }
        return rawSql(dataFromCursor)(this.db, transformBlobs !== null && transformBlobs !== void 0 ? transformBlobs : this.transformBlobs)(query, params);
    }
    async selectArray(query, params, transformBlobs) {
        if (this.threading) {
            return this.sendMessageToWorker({
                db: this.db,
            }, {
                callName: 'selectArray',
                args: [query, params],
            });
        }
        return rawSql(arrayFromCursor)(this.db, transformBlobs !== null && transformBlobs !== void 0 ? transformBlobs : this.transformBlobs)(query, params);
    }
    async each(query, params, callback, complete, transformBlobs) {
        return eachRaw(dataFromCursor)(this.db, transformBlobs !== null && transformBlobs !== void 0 ? transformBlobs : this.transformBlobs)(query, params, callback, complete);
    }
    async transaction(action) {
        return transactionRaw(this.db, action);
    }
}
//# sourceMappingURL=sqlitedatabase.android.js.map