"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConverterIOS = exports.ConverterAndroid = void 0;
const fs = require("fs");
const path = require("path");
const plist = require("simple-plist");
const converter_common_1 = require("./converter.common");
const shorthash = require("shorthash");
const flagsWidthPrecisionLength = '([-+0#])?(\\d+|\\*)?(\\.(\\d+|\\*))?(hh?|ll?|L|z|j|t|q)?';
const parameterFlagsWidthPrecisionLength = `(\\d+\\$)?${flagsWidthPrecisionLength}`;
const types = '[diufFeEgGxXoscpaA]';
const isEndingWithPercentSign = function (subject) {
    return /(?:^|[^%])(?:%%)*%$/.test(subject);
};
const encodeKey = function (key) {
    return key.match(/^[_a-zA-Z]\w*$/) ? key : `_${key.replace(/[^\w]/g, '_')}_${shorthash.unique(key)}`;
};
const replace = function (find, replace, subject) {
    return subject.replace(new RegExp('(' + find.map((i) => i.replace(/[.?*+^$[\]\\(){}|-]/g, '\\$&')).join('|') + ')', 'g'), (match) => replace[find.indexOf(match)]);
};
const convertPlaceholders = function (subject) {
    return numberPlaceholders(escapeSinglePercentSign(subject));
};
const numberPlaceholders = function (subject) {
    let index = 0;
    return replacePlaceholder(new RegExp(`%(${flagsWidthPrecisionLength}${types})`, 'g'), (_, $1) => `%${++index}$${$1}`, subject);
};
const escapeSinglePercentSign = function (subject) {
    return subject.replace(new RegExp(`(^|[^%])(%)(?!(%|${parameterFlagsWidthPrecisionLength}${types}))`, 'g'), '$1%%');
};
const convertStringSignToAtSign = function (subject) {
    return replacePlaceholder(new RegExp(`(%${parameterFlagsWidthPrecisionLength})s`, 'g'), (_, $1) => `${$1}@`, subject);
};
const replacePlaceholder = function (find, replace, subject) {
    let match = null;
    while (null !== (match = find.exec(subject))) {
        const previousString = subject.substring(0, match.index);
        if (isEndingWithPercentSign(previousString)) {
            continue;
        }
        const replacedString = replace(...match);
        find.lastIndex += replacedString.length - match[0].length;
        subject = previousString + replacedString + subject.substring(match.index + match[0].length);
    }
    return subject;
};
class ConverterAndroid extends converter_common_1.ConverterCommon {
    constructor(dataProvider, logger, platformData, projectData, androidResourcesMigrationService, environmentName) {
        super(dataProvider, logger, platformData, projectData, environmentName);
        this.environmentName = environmentName;
        if (androidResourcesMigrationService.hasMigrated(projectData.appResourcesDirectoryPath)) {
            this.appResourcesDirectoryPath = path.join(this.appResourcesDirectoryPath, 'src', 'main', 'res');
        }
    }
    encodeValue(value) {
        return '"' + replace(["'", '"', '\\', '\n', '\r', '\t', '<', '&'], ["\\'", '\\"', '\\\\', '\\n', '\\r', '\\t', '&lt;', '&amp;'], convertPlaceholders(value)) + '"';
    }
    cleanObsoleteResourcesFiles(resourcesDirectory, languages) {
        fs.readdirSync(resourcesDirectory)
            .filter((fileName) => {
            const match = /^values-(.+)$/.exec(fileName);
            return match && !languages.has(match[1].replace(/^(.+?)-r(.+?)$/, '$1-$2'));
        })
            .map((fileName) => {
            return path.join(resourcesDirectory, fileName);
        })
            .filter((filePath) => {
            return fs.statSync(filePath).isDirectory();
        })
            .forEach((lngResourcesDir) => {
            const resourceFilePath = path.join(lngResourcesDir, 'strings.xml');
            this.removeFileIfExists(resourceFilePath);
            this.removeDirectoryIfEmpty(lngResourcesDir);
        });
        return this;
    }
    readRules() {
        const fileName = "environment-rules." + 'android' + ".json";
        const ruleFile = path.join(this.projectData.projectDir, fileName);
        if (fs.existsSync(ruleFile)) {
            this.logger.debug("Environment Rules found, reading contents");
            return JSON.parse(fs.readFileSync(ruleFile).toString());
        }
        else {
            this.logger.fatal("1111 Environment Rules File does not exist, Skipping....");
            return;
        }
    }
    createLanguageResourcesFiles(language, isDefaultLanguage, i18nEntries) {
        let languageResourcesDir = path.join(this.appResourcesDirectoryPath, `values${isDefaultLanguage ? '' : `-${language.replace(/^(.+?)-(.+?)$/, '$1-r$2')}`}`);
        let multiEnvironmentName = [];
        if (this.environmentName) {
            const multEnvironmentRules = this.readRules();
            if (multEnvironmentRules) {
                multiEnvironmentName = multEnvironmentRules.environments.map(environment => environment.name);
            }
        }
        const canCreateFile = multiEnvironmentName.some(envName => languageResourcesDir.includes('.' + envName));
        if (!canCreateFile) {
            this.createDirectoryIfNeeded(languageResourcesDir);
        }
        let strings = '<?xml version="1.0" encoding="utf-8"?>\n<resources>\n';
        this.encodeI18nEntries(i18nEntries).forEach((encodedValue, encodedKey) => {
            strings += `  <string name="${encodedKey}">${encodedValue}</string>\n`;
        });
        strings += '</resources>\n';
        const resourceFilePath = path.join(languageResourcesDir, 'strings.xml');
        if (multiEnvironmentName.length > 0) {
            multiEnvironmentName.map(multEnvironmentName => {
                languageResourcesDir = languageResourcesDir.replace("/\.default." + multEnvironmentName + "/g", '');
                languageResourcesDir = languageResourcesDir.replace("/\." + multEnvironmentName + "/g", '');
            });
        }
        if (!canCreateFile) {
            this.writeFileSyncIfNeeded(resourceFilePath, strings);
        }
        return this;
    }
    encodeI18nEntries(i18nEntries) {
        const encodedI18nEntries = new Map();
        i18nEntries.forEach((value, key) => {
            const encodedKey = encodeKey(key);
            const encodedValue = this.encodeValue(value);
            encodedI18nEntries.set(encodedKey, encodedValue);
            if (key === 'app.name') {
                encodedI18nEntries.set('app_name', encodedValue);
                encodedI18nEntries.set('title_activity_kimera', encodedValue);
            }
        });
        return encodedI18nEntries;
    }
}
exports.ConverterAndroid = ConverterAndroid;
class ConverterIOS extends converter_common_1.ConverterCommon {
    encodeValue(value) {
        return replace(['"', '\\', '\n', '\r', '\t'], ['\\"', '\\\\', '\\n', '\\r', '\\t'], convertStringSignToAtSign(convertPlaceholders(value)));
    }
    cleanObsoleteResourcesFiles(resourcesDirectory, languages) {
        fs.readdirSync(resourcesDirectory)
            .filter((fileName) => {
            const match = /^(.+)\.lproj$/.exec(fileName);
            return match && !languages.has(match[1]);
        })
            .map((fileName) => {
            return path.join(resourcesDirectory, fileName);
        })
            .filter((filePath) => {
            return fs.statSync(filePath).isDirectory();
        })
            .forEach((lngResourcesDir) => {
            ['InfoPlist.strings', 'Localizable.strings'].forEach((fileName) => {
                const resourceFilePath = path.join(lngResourcesDir, fileName);
                this.removeFileIfExists(resourceFilePath);
            });
            this.removeDirectoryIfEmpty(lngResourcesDir);
        });
        return this;
    }
    createLanguageResourcesFiles(language, isDefaultLanguage, i18nEntries) {
        const infoPlistStrings = new Map();
        i18nEntries.forEach((value, key) => {
            if (key === 'app.name') {
                infoPlistStrings.set('CFBundleDisplayName', value);
            }
            else if (key.startsWith('ios.info.plist.')) {
                infoPlistStrings.set(key.substr(15), value);
            }
        });
        const languageResourcesDir = path.join(this.appResourcesDirectoryPath, `${language}.lproj`);
        this.createDirectoryIfNeeded(languageResourcesDir).writeStrings(languageResourcesDir, 'Localizable.strings', i18nEntries).writeStrings(languageResourcesDir, 'InfoPlist.strings', infoPlistStrings);
        if (isDefaultLanguage) {
            infoPlistStrings.set('CFBundleDevelopmentRegion', language);
            this.writeInfoPlist(infoPlistStrings);
        }
        return this;
    }
    encodeI18nEntries(i18nEntries) {
        const encodedI18nEntries = new Map();
        i18nEntries.forEach((value, key) => {
            const encodedKey = encodeKey(key);
            const encodedValue = this.encodeValue(value);
            encodedI18nEntries.set(encodedKey, encodedValue);
        });
        return encodedI18nEntries;
    }
    writeStrings(languageResourcesDir, resourceFileName, i18nEntries) {
        let content = '';
        this.encodeI18nEntries(i18nEntries).forEach((encodedValue, encodedKey) => {
            content += `"${encodedKey}" = "${encodedValue}";\n`;
        });
        const resourceFilePath = path.join(languageResourcesDir, resourceFileName);
        this.writeFileSyncIfNeeded(resourceFilePath, content);
        return this;
    }
    writeInfoPlist(infoPlistValues) {
        const resourceFilePath = path.join(this.appResourcesDirectoryPath, 'Info.plist');
        if (!fs.existsSync(resourceFilePath)) {
            this.logger.warn(`'${resourceFilePath}' doesn't exists: unable to set default language`);
            return this;
        }
        const data = plist.readFileSync(resourceFilePath);
        let resourceChanged = false;
        infoPlistValues.forEach((value, key) => {
            if (!data.hasOwnProperty(key) || data[key] !== value) {
                data[key] = value;
                resourceChanged = true;
            }
        });
        if (resourceChanged) {
            plist.writeFileSync(resourceFilePath, data);
        }
    }
}
exports.ConverterIOS = ConverterIOS;
