import { merge } from 'webpack-merge';
import Config from 'webpack-chain';
import webpack from 'webpack';
import { determineProjectFlavor } from './helpers/flavor';
import { error, info } from './helpers/log';
export interface IWebpackEnv {
    [name: string]: any;
    env?: string;
    appPath?: string;
    appResourcesPath?: string;
    appComponents?: string[];
    nativescriptLibPath?: string;
    android?: boolean;
    ios?: boolean;
    platform?: string;
    sourceMap?: string | boolean;
    production?: boolean;
    report?: boolean;
    hmr?: boolean;
    verbose?: boolean;
    replace?: string[] | string;
}
/**
 * The default flavor specific configs
 */
export declare const defaultConfigs: {
    base: typeof import("./configuration/base").default;
    angular: typeof import("./configuration/angular").default;
    javascript: typeof import("./configuration/javascript").default;
    react: typeof import("./configuration/react").default;
    svelte: typeof import("./configuration/svelte").default;
    typescript: typeof import("./configuration/typescript").default;
    vue: typeof import("./configuration/vue").default;
};
/**
 * Utilities to simplify various tasks
 */
export declare const Utils: {
    merge: typeof merge;
    addCopyRule: typeof import("./helpers/copyRules").addCopyRule;
    removeCopyRule: typeof import("./helpers/copyRules").removeCopyRule;
    applyFileReplacements: typeof import("./helpers/fileReplacements").applyFileReplacements;
    config: {
        getValue: typeof import("./helpers/config").getValue;
    };
    dependencies: {
        getAllDependencies: typeof import("./helpers/dependencies").getAllDependencies;
        hasDependency: typeof import("./helpers/dependencies").hasDependency;
        getDependencyPath: typeof import("./helpers/dependencies").getDependencyPath;
    };
    flavor: {
        determineProjectFlavor: typeof determineProjectFlavor;
    };
    host: {
        getIPS: typeof import("./helpers/host").getIPS;
    };
    log: {
        error: typeof error;
        info: typeof info;
        warn: typeof import("./helpers/log").warn;
        warnOnce: typeof import("./helpers/log").warnOnce;
    };
    platform: {
        addPlatform: typeof import("./helpers/platform").addPlatform;
        getAbsoluteDistPath: typeof import("./helpers/platform").getAbsoluteDistPath;
        getDistPath: typeof import("./helpers/platform").getDistPath;
        getEntryDirPath: typeof import("./helpers/platform").getEntryDirPath;
        getEntryPath: typeof import("./helpers/platform").getEntryPath;
        getPlatform: typeof import("./helpers/platform").getPlatform;
        getPlatformName: typeof import("./helpers/platform").getPlatformName;
    };
    project: {
        getProjectFilePath: typeof import("./helpers/project").getProjectFilePath;
        getProjectRootPath: typeof import("./helpers/project").getProjectRootPath;
        getPackageJson: typeof import("./helpers/project").getPackageJson;
    };
    virtualModules: {
        addVirtualEntry: typeof import("./helpers/virtualModules").addVirtualEntry;
        addVirtualModule: typeof import("./helpers/virtualModules").addVirtualModule;
    };
};
/**
 * Initialize @nativescript/webpack with the webpack env.
 * Must be called first.
 *
 * @param _env The webpack env
 */
export declare function init(_env: IWebpackEnv): void;
/**
 * Explicitly specify the base config to use.
 * Calling this will opt-out from automatic flavor detection.
 *
 * Useful when the flavor cannot be detected due to the project structure
 * for example in a custom monorepo.
 *
 * @param config Name of the base config to use.
 */
export declare function useConfig(config: keyof typeof defaultConfigs | false): void;
/**
 * Add a new function to be called when building the internal config using webpack-chain.
 *
 * @param chainFn A function that accepts the internal chain config, and the current environment
 * @param options Optional options to control the order in which the chain function should be applied.
 */
export declare function chainWebpack(chainFn: (config: Config, env: IWebpackEnv) => any, options?: {
    order?: number;
}): void;
/**
 * Merge an object into the resolved chain config.
 *
 * @param mergeFn An object or a function that optionally returns an object (can mutate the object directly and return nothing)
 */
export declare function mergeWebpack(mergeFn: (config: Partial<webpack.Configuration>, env: IWebpackEnv) => any | Partial<webpack.Configuration>): void;
/**
 * Resolve a new instance of the internal chain config with all chain functions applied.
 */
export declare function resolveChainableConfig(): Config;
/**
 * Resolve a "final" configuration that has all chain functions and merges applied.
 *
 * @param chainableConfig Optional chain config to use.
 */
export declare function resolveConfig(chainableConfig?: Config): webpack.Configuration;
